import importlib
import pkgutil

from layered_py.config_loader import Config

config = Config()
AUTOLOAD_PACKAGES = config.get_folder_array()

def load_all_modules():
    for pkg_name in AUTOLOAD_PACKAGES:
        try:
            pkg = importlib.import_module(pkg_name)
        except ModuleNotFoundError:
            continue  # Skip missing folders

        for module_info in pkgutil.walk_packages(pkg.__path__, prefix=pkg.__name__ + "."):
            importlib.import_module(module_info.name)


import os
from constants.templates import get_service_template, get_domain_template, get_repository_template, \
    get_presentation_template
from layered_py.config_loader import Config
from layered_py.decorators import inject

config = Config()

def create_service(service_name):
    service_template = get_service_template(service_name)

    # Ensure the 'services' directory exists
    directory = config.get('service_destination')
    if not os.path.exists(directory):
        os.makedirs(directory)

    file_path = f"{directory}/{service_name}.py"

    with open(file_path, "w") as file:
        file.write(service_template)

    print(f"Service '{service_name}' created at {file_path}")


def create_domain(domain_name):
    domain_template = get_domain_template(domain_name)

    directory = config.get('domain_destination')
    if not os.path.exists(directory):
        os.makedirs(directory)

    file_path = f"{directory}/{domain_name}.py"

    with open(file_path, "w") as file:
        file.write(domain_template)

    print(f"Service '{domain_name}' created at {file_path}")


def create_repository(repository_name):
    repository_template = get_repository_template(repository_name)

    directory = config.get('repository_destination')
    if not os.path.exists(directory):
        os.makedirs(directory)

    file_path = f"{directory}/{repository_name}.py"

    with open(file_path, "w") as file:
        file.write(repository_template)

    print(f"Service '{repository_name}' created at {file_path}")


def create_presentation(presentation_name):
    presentation_template = get_presentation_template(presentation_name)

    directory = config.get('presentation_destination')
    if not os.path.exists(directory):
        os.makedirs(directory)

    file_path = f"{directory}/{presentation_name}.py"

    with open(file_path, "w") as file:
        file.write(presentation_template)

    print(f"Service '{presentation_name}' created at {file_path}")



import os
import yaml
from layered_py.decorators import register

DEFAULTS = {
    'service_destination': 'services',
    'domain_destination': 'domain',
    'repository_destination': 'repository',
    'presentation_destination': 'presentation'
}


def _load_config(path):
    if not os.path.exists(path):
        raise FileNotFoundError(f"Config file not found at {path}")

    with open(path, 'r') as file:
        return yaml.safe_load(file)


@register('Config')
class Config:
    def __init__(self):
        self.config = DEFAULTS.copy()

        if os.path.exists('layeredpy_config.yml'):
            self.config.update(_load_config('layeredpy_config.yml'))

    def get(self, key):
        return self.config.get(key)

    def get_folder_array(self):
        return {
            self.get('service_destination'),
            self.get('domain_destination'),
            self.get('repository_destination'),
            self.get('presentation_destination')
        }


def get_service_template(service_name):
    pascal_name = to_pascal_case(service_name)

    service_template = f"""from layered_py.service import Service
from layered_py.decorators import register

@register(singleton=True)
class {pascal_name}(Service):
    def setup(self):
        super().__init__()

    def handle(self):
        raise NotImplementedError
        """
    return service_template

def get_domain_template(domain_name):
    pascal_name = to_pascal_case(domain_name)

    domain_template = f"""from layered_py.domain import BaseDomain
from layered_py.decorators import register
    
@register(singleton=True)
class {pascal_name}(BaseDomain):
    def __init__(self):
        super().__init__()
        """
    return domain_template

def get_repository_template(repository_name):
    pascal_name = to_pascal_case(repository_name)

    repository_template = f"""from layered_py.repository import BaseRepository
from layered_py.decorators import register

@register(singleton=True)
class {pascal_name}(BaseRepository):
    def __init__(self):
        super().__init__()
        """
    return repository_template

def get_presentation_template(presentation_name):
    pascal_name = to_pascal_case(presentation_name)

    presentation_template = f"""from layered_py.presentation import BasePresentation
from layered_py.decorators import register

@register(singleton=True)
class {pascal_name}(BasePresentation):
    def __init__(self):
        super().__init__()
        """
    return presentation_template

def to_pascal_case(snake_str):
    return ''.join(word.capitalize() for word in snake_str.split('_'))



class Container:
    def __init__(self):
        self._providers = {}
        self._singletons = {}

    def register(self, key, provider, *, singleton=False):
        self._providers[key] = {
            "provider": provider,
            "singleton": singleton
        }

    def resolve(self, key):
        if key not in self._providers:
            raise KeyError(f"No provider registered for key: '{key}'")

        entry = self._providers[key]

        if entry["singleton"]:
            if key not in self._singletons:
                self._singletons[key] = entry["provider"]()
                return self._singletons[key]
            else:
                return entry["provider"]()


import inspect
from functools import wraps
from inspect import isclass
from .di import Container

container = Container()

def register(key=None, singleton=False):
    def wrapper(provider):
        reg_key = key or provider.__name__
        if isclass(provider):
            container.register(reg_key, lambda: provider(), singleton=singleton)
        else:
            container.register(reg_key, provider, singleton=singleton)
        return provider
    return wrapper

def inject(func):
    sig = inspect.signature(func)

    @wraps(func)
    def wrapper(*args, **kwargs):
        bound = sig.bind_partial(*args, **kwargs)

        for name, param in sig.parameters.items():
            if name not in bound.arguments:
                if param.annotation != inspect.Parameter.empty:
                    try:
                        kwargs[name] = container.resolve(param.annotation.__name__)
                    except KeyError:
                        pass
                if name not in kwargs:
                    kwargs[name] = container.resolve(name)
        return func(*args, **kwargs)
    return wrapper


from layered_py.shared import create_service, create_domain, create_repository, create_presentation


def generate(name):
    formatted_name = name.lower()

    create_service(f"{formatted_name}_service")
    create_domain(f"{formatted_name}")
    create_repository(f"{formatted_name}_repository")
    create_presentation(f"{formatted_name}_routes")


import sys

from layered_py.generator import generate
from layered_py.shared import create_service, create_domain, create_repository

commands = ['createService', 'createDomain', 'createRepository', 'generate']
def main():
    if len(sys.argv) != 3 or sys.argv[1] not in commands:
        print("Possible commands are:")
        for command in commands:
            print(command)
        sys.exit(1)

    command, name = sys.argv[1], sys.argv[2]
    match command:
        case "createService":
            create_service(name)
        case "createDomain":
            create_domain(name)
        case "createRepository":
            create_repository(name)
        case "generate":
            generate(name)

if __name__ == "__main__":
    main()


class Service:
    def __init__(self, *args, **kwargs):
        self.__dependencies = kwargs
        self.setup()

    def setup(self):
        pass

    def handle(self):
        raise NotImplementedError()